package de.w665.sharepulse.service;

import de.w665.sharepulse.db.repo.UserRepository;
import de.w665.sharepulse.model.User;
import io.jsonwebtoken.*;
import io.jsonwebtoken.security.Keys;
import jakarta.annotation.PostConstruct;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import javax.crypto.SecretKey;
import java.util.Base64;
import java.util.Date;
import java.util.Optional;

@Slf4j
@Service
public class AuthenticationService {

    private final UserRepository userRepository;

    @Value("${secureapi.jwt.secret}")
    private String secretString;
    @Value("${secureapi.jwt.expiration}")
    private long expirationTime; // in milliseconds
    private SecretKey secretKey;

    private final BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public AuthenticationService(UserRepository userRepository) {
        this.userRepository = userRepository;
    }

    @PostConstruct
    public void init() {
        log.debug("Initializing secret key");
        byte[] encodedKey = Base64.getEncoder().encode(secretString.getBytes()); // encode the secret key
        this.secretKey = Keys.hmacShaKeyFor(encodedKey);
    }

    public String authenticate(String username, String password, long... expirationTime/*FOR TESTING VALIDITY*/) {
        if(expirationTime.length > 0) {
            this.expirationTime = expirationTime[0];
        }
        Optional<User> user = userRepository.retrieveUserByUsername(username);
        if (user.isPresent() && passwordEncoder.matches(password, user.get().getPassword())) {
            return generateToken(username);
        }
        return null;
    }

    private String generateToken(String username) {
        long nowMillis = System.currentTimeMillis();
        Date now = new Date(nowMillis);
        Date expiryDate = new Date(nowMillis + expirationTime);

        return Jwts.builder()
                .subject(username)
                .issuedAt(now)
                .expiration(expiryDate)
                .signWith(secretKey)
                .compact();
    }

    public boolean validateToken(String token) {
        try {
            Jwt<?,?> jwt = Jwts.parser().verifyWith(secretKey).build().parseSignedClaims(token);
            Claims claims = (Claims) jwt.getPayload();
            return !claims.getExpiration().before(new Date()); // Checks if the token is expired too
        } catch (Exception e) {
            return false;
        }
    }
}
